package com.yanzuoguang.cloud.aop;

import com.yanzuoguang.cloud.CloudContans;
import com.yanzuoguang.util.contants.ResultConstants;
import com.yanzuoguang.util.exception.ExceptionHelper;
import com.yanzuoguang.util.helper.StringHelper;
import com.yanzuoguang.util.log.Log;
import com.yanzuoguang.util.vo.LogVo;
import com.yanzuoguang.util.vo.ResponseResult;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

/**
 * LogsAspect（接口请求日志切面）
 *
 * @author: Kang
 * @time: 2018年04月25日 11:43
 */
@Aspect
@Component
public class MqAspect extends BaseRequestAspect {

    private static final Logger logger = LoggerFactory.getLogger(MqAspect.class);

    /**
     * exec aop point aspect
     */
    @Pointcut("execution(* *..mq..*Consumer.*(..))")
    public void mqAspect() {
    }

    /**
     * 执行环形切面
     *
     * @param joinPoint
     * @return
     */
    @Around(value = "mqAspect()")
    public Object requestWebAround(ProceedingJoinPoint joinPoint) throws Throwable {
        Log.threadBegin();
        long start = System.currentTimeMillis();
        String name = joinPoint.getSignature().getName();
        logger.info("[ {} ] request params is {}", name, joinPoint.getArgs());
        try {
            Object ret = executeMethod(joinPoint, name);
            long end = System.currentTimeMillis();
            logger.info("[ {} ] time ({})ms,  result is {}", name, (end - start), ret);
            return ret;
        } catch (Exception e) {
            long end = System.currentTimeMillis();
            logger.error("[ {} ] time ({})ms,  is error {}", name, (end - start), e);

            // 消费消息出错
            ResponseResult responseResult = ExceptionHelper.getError(e);
            saveInterLogs(joinPoint, responseResult);
            Log.error(joinPoint.getSignature().getDeclaringType(), e);

            throw e;
        } finally {
            Log.threadCommit();
        }
    }

    private Object executeMethod(ProceedingJoinPoint joinPoint, String name) throws Throwable {
        return joinPoint.proceed();
    }

    /**
     * deal with inter api logs
     *
     * @param joinPoint      请求参数
     * @param responseResult 返回参数
     */
    private void saveInterLogs(ProceedingJoinPoint joinPoint, ResponseResult responseResult) {
        // 日志请求不记录,防止死循环递归
        if (applicationName.indexOf(CloudContans.LOG_MODULE) >= 0) {
            return;
        }
        // 正常请求不记录
        if (!logAll && responseResult.getCode() == ResultConstants.SUCCESS) {
            return;
        }
        LogVo logVo = initLogInterVo(StringHelper.EMPTY, joinPoint, responseResult);
        addLog(logVo);
    }
}
